<?php defined('SYSPATH') or die('No direct script access.');

class Model_Account extends Model
{
    public static function setHashKey($key) {
        # TODO: not implemented
    }

    public static function getMaintenanceInfo() {
        $config = Model_System::loadConfig("general.json");

        $result['active'] = $config->Maintenance->active;
        $result['return'] = $config->Maintenance->return;
        return $result;
    }

    public static function checkAuth($login, $password) {
        $sql = "SELECT `SignIn`(:login, :pass) AS `ID`;";
        return DB::query(Database::SELECT, $sql)
            ->param(':login', $login)
            ->param(':pass', $password)
            ->execute()->get('ID');
    }


    private static function generateActivationCode() {
        $activationCode = Text::random('ABDCEFGHJKLMNPQRSTUVWXYZ123456789', 10);
        return UTF8::strtoupper($activationCode);
    }

    public static function createTeacher($lastName, $firstName, $secondName, $degreeID, $departmentID) {
        $code = self::generateActivationCode();

        $sql = "SELECT `CreateTeacher`(:last, :first, :second, :degree, :department, :code) AS `teacherID`";
        $response = DB::query(Database::SELECT, $sql)
            ->parameters([
                ':last'       => $lastName,
                ':first'      => $firstName,
                ':second'     => $secondName,
                ':degree'     => $degreeID,
                ':department' => $departmentID,
                ':code'       => $code,
            ])->execute()->get('teacherID');

        return $response == -1 ? -1 : $code;
    }

    public static function createTeacherByDepName($lastName, $firstName, $secondName, $department, $facultyID) {
        if ($department == '') return -1;

        $code = self::generateActivationCode();

        $sql = "SELECT `CreateTeacherByDepName`(:last, :first, :second, :department, :faculty, :code) AS `UserID`;";
        $response = DB::query(Database::SELECT, $sql)
            ->parameters([
                ':last'       => trim($lastName),
                ':first'      => trim($firstName),
                ':second'     => trim($secondName),
                ':department' => $department,
                ':faculty'    => $facultyID,
                ':code'       => $code,
            ])->execute()->get('UserID');

        return $response == -1 ? -1 : $code;
    }

    public static function createStudent($lastName, $firstName, $secondName, $grade, $groupNum, $facultyID, $semesterID = null) {
        $code = self::generateActivationCode();

        if (!$semesterID) $semesterID = Model_System::$SemesterID;

        $sql = "SELECT `CreateStudent`(:last, :first, :second, :grade, :group, :faculty, :code, :semester) AS `UserID`";

        $response = DB::query(Database::SELECT, $sql)
            ->parameters([
                ':last'     => trim($lastName),
                ':first'    => trim($firstName),
                ':second'   => trim($secondName),
                ':grade'    => $grade,
                ':group'    => $groupNum,
                ':faculty'  => $facultyID,
                ':code'     => $code,
                ':semester' => $semesterID
            ])->execute()->get('UserID');

        return $response == -1 ? -1 : $code;
    }

    public static function createStudentEx($lastName, $firstName, $secondName, $gradeNum, $groupNum, $degree, $specialization, $facultyID, $semesterID = null) {
        $code = self::generateActivationCode();

        if (!$semesterID) $semesterID = Model_System::$SemesterID;

        $sql = "SELECT `CreateStudentEx`(:last, :first, :second, :grade, :group, :degree, :spec, :faculty, :code, :semester) AS `UserID`;";
        $response = DB::query(Database::SELECT, $sql)
            ->parameters([
                ':last'     => trim($lastName),
                ':first'    => trim($firstName),
                ':second'   => trim($secondName),
                ':grade'    => $gradeNum,
                ':group'    => $groupNum,
                ':degree'   => $degree,
                ':spec'     => $specialization,
                ':faculty'  => $facultyID,
                ':code'     => $code,
                ':semester' => $semesterID
            ])->execute()->get('UserID');

        return $response == -1 ? -1 : $code;
    }


    public static function getUserInfo($id, $semesterID = null) {
        $semesterID = $semesterID ? $semesterID : User::instance()->SemesterID;
        $sql = "CALL `GetFullInfo`(:id, :semester);";
        return DB::query(Database::SELECT, $sql)
            ->param(':id', (int) $id)
            ->param(':semester', (int) $semesterID)
            ->execute()[0];
    }


    # TODO: deprecated (see getUserInfo)
    public static function getPersonalInfo($id, $semesterID = null) {
        $semesterID = $semesterID ? $semesterID : User::instance()->SemesterID;
        $sql = "CALL `GetPersonalInfo`(:id, :semester);";
        return DB::query(Database::SELECT, $sql)
            ->param(':semester', (int) $semesterID)
            ->param(':id', (int) $id)
            ->execute()[0];
    }

    # TODO: deprecated (see getUserInfo)
    public static function getAccountInfo($id) {
        return DB::query(Database::SELECT, "CALL GetAccountInfo(:id);")
            ->param(':id', (int) $id)->execute()[0];
    }

    /**
     * @param int    $id
     * @param string $value
     * @param string $type 'email', 'login' or 'password'
     * @return int
     */
    public static function changeAccountData($id, $value, $type) {
        $sql = "SELECT `ChangeAccountData`(:account, :value, :type) AS Num;";
        return DB::query(Database::SELECT, $sql)
            ->parameters([
                ':account' => $id,
                ':value'   => $value,
                ':type'    => $type,
            ])->execute()->get('Num');
    }

    /**
     * @param string $data
     * @param string $type 'login','email' or 'code'
     * @return int
     */
    public static function checkAccountExistence($data, $type) {
        $sql = "SELECT `CheckAccountExistence`(:data, :type) AS Num;";
        return DB::query(Database::SELECT, $sql)
            ->param(':data', $data)
            ->param(':type', $type)
            ->execute()->get('Num');
    }

    public static function isActivationCodeValid($code) {
        $sql = "SELECT `CheckAccountExistence`(:acode, 'code') AS Num;";
        $res = DB::query(Database::SELECT, $sql)
            ->param(':acode', $code)
            ->execute()->get('Num');
        return $res == 1;
    }

    public static function createRecoveryToken($email, $token) {
        $sql = "SELECT `CreateRecoveryToken`(:email, :token) AS UserName;";
        return DB::query(Database::SELECT, $sql)
            ->param(':email', $email)
            ->param(':token', $token)
            ->execute()->get('UserName');
    }

    public static function getRecoveryInfoByEMail($email) {
        $sql = "CALL GetRecoveryInfoByEMail(:email);";
        return DB::query(Database::SELECT, $sql)
            ->param(':email', $email)
            ->execute();
    }

    public static function getRecoveryInfoByToken($token) {
        $sql = "CALL GetRecoveryInfoByToken(:token);";
        return DB::query(Database::SELECT, $sql)
            ->param(':token', $token)->execute();
    }

    public static function useRecoveryToken($token) {
        $sql = "SELECT `UseRecoveryToken`(:token) AS Num;";
        return DB::query(Database::SELECT, $sql)
            ->param(':token', $token)
            ->execute()->get('Num');
    }

    public static function activateAccount($login, $password, $email, $code) {
        $sql = "SELECT `ActivateAccount`(:code, :login, :email, :pass) AS `Num`;";
        return DB::query(Database::SELECT, $sql)
            ->parameters([
                ':code'  => $code,
                ':login' => $login,
                ':email' => $email,
                ':pass'  => $password,
            ])->execute()->get('Num');
    }

}