<?php defined('SYSPATH') or die('No direct script access.');

/**
 * @property-read $ID          int
 * @property      $AuthorID    int
 * @property      $GradeID     int
 * @property      $GradeNum    int
 * @property      $FacultyID   int
 * @property      $FacultyName string
 * @property      $Degree      string  bachelor / master
 * @property      $SubjectID   int
 * @property      $SubjectName string
 * @property      $SubjectAbbr string
 * @property      $SemesterID  int
 * @property      $Lectures    int
 * @property      $Practice    int
 * @property      $Labs        int
 * @property      $Type        string
 * @property      $Subtype     string
 * @property      $IsLocked    bool
 * @property      $IsBonus     bool
 * @property      $Milestone   int
 */
class Model_Discipline extends Model_Container
{
    const EXAM = 'exam';
    const CREDIT = 'credit';
    const GRADING_CREDIT = 'grading_credit';

    /**
     * @param $id int discipline id
     * @return array data from <tt>view_disciplines</tt> table
     * @throws HTTP_Exception if discipline does not exist
     */
    protected function getRawData($id) {
        $sql = "CALL `Discipline_GetInfo`(:id)";
        $info = DB::query(Database::SELECT, $sql)
            ->param(':id', $id)->execute();

        if ($info->count() == 0)
            throw new InvalidArgumentException(Error::DISCIPLINE_NOT_FOUND);
        return $info->offsetGet(0);
    }

    /**
     * Creation of new discipline from raw data.
     * @return Model_Helper_DisciplineBuilder
     * todo: may be we need a factory pattern
     */
    static public function make() {
        return new Model_Helper_DisciplineBuilder();
    }

    /**
     * Create new discipline in db, based on $data.
     */
    protected function create() {
        $sql = "SELECT `Discipline_Create`(AuthorID, GradeID, SubjectID, Type, Lectures, Practice, Labs, FacultyID, SemesterID, Subtype) AS `Num`;";

        $this->data[self::$ID_FIELD] = DB::query(Database::SELECT, $sql)
            ->parameters($this->data)
            ->execute()[0]['Num'];
    }

    public function delete() {
        $sql = "SELECT `Discipline_Delete`(:id);";
        DB::query(Database::SELECT, $sql)->param(':id', $this->ID)->execute();
    }

    // todo: should return Model_Group[]
    public function getGroups() {
        $sql = "CALL `GetGroupsForDiscipline`(:id);";
        return DB::query(Database::SELECT, $sql)
            ->param(':id', $this->ID)
            ->execute()->as_array();
    }

    // todo: should return Model_Teacher[]
    public function getTeachers() {
        $sql = "CALL `GetTeachersForDiscipline`(:id)";
        return DB::query(Database::SELECT, $sql)
            ->param(':id', $this->ID)->execute();
    }

    public function bind(Model_Teacher $teacher) {
        if ($this->ID == $teacher->ID)
            return;

        $sql = "SELECT `Discipline_BindTeacher`(:id, :teacher)";
        DB::query(Database::SELECT, $sql)
            ->param(':teacher', $teacher->ID)
            ->param(':id', $this->ID)
            ->execute();
    }

    public function unbind(Model_Teacher $teacher) {
        if ($this->ID == $teacher->ID)
            return;

        $sql = "SELECT `Discipline_UnbindTeacher`(:id, :teacher)";
        DB::query(Database::SELECT, $sql)
            ->param(':teacher', $teacher->ID)
            ->param(':id', $this->ID)
            ->execute();
    }

    /**
     * Bind teacher and delegate him the discipline.
     * @param Model_Teacher $teacher
     */
    public function delegateTo(Model_Teacher $teacher) {
        if ($this->ID == $teacher->ID)
            return;

        $sql = "SELECT `Discipline_Delegate`(:id, :teacher)";
        DB::query(Database::SELECT, $sql)
            ->param(':teacher', $teacher->ID)
            ->param(':id', $this->ID)
            ->execute();
    }


    public static function changeGrade($disciplineID, $teacherID, $grade) {
        $sql = "SELECT `ChangeDisciplineGrade`(:teacher, :discipline, :grade) AS `Num`";
        return DB::query(Database::SELECT, $sql)
            ->parameters([
                ':teacher'    => $teacherID,
                ':discipline' => $disciplineID,
                ':grade'      => $grade,
            ])->execute();
    }

    /**
     * @param $teacherID    int
     * @param $disciplineID int
     * @param $milestone    int
     * @return Database_Result
     */
    public static function setMilestone($teacherID, $disciplineID, $milestone) {
        $sql = "SELECT `RestrictAfterMilestone`(:teacher, :discipline, milestone) AS `Num`";
        return DB::query(Database::SELECT, $sql)
            ->parameters([
                ':teacher'    => $teacherID,
                ':discipline' => $disciplineID,
                ':milestone'  => $milestone,
            ])->execute();
    }

    /**
     * @param $facultyID  int
     * @param $semesterID int
     * @return Database_Result
     */
    public static function getMilestone($facultyID, $semesterID) {
        $sql = "SELECT `GetMilestone`(:faculty, :semester) AS `Num`";
        return DB::query(Database::SELECT, $sql)
            ->param(':faculty', $facultyID)
            ->param(':semester', $semesterID)
            ->execute();
    }


    /**
     * @param $teacherID  int
     * @param $facultyID  int
     * @param $milestone  int
     * @param $semesterID int
     * @return Database_Result
     */
    public static function setMilestoneForCredits($teacherID, $facultyID, $milestone, $semesterID = null) {
        $semesterID = $semesterID ? $semesterID : User::instance()->SemesterID;
        $sql = "SELECT `RestrictAfterMilestoneForCredits`(:teacherID, :facultyID, :milestone, :semesterID) AS `Num`";
        return DB::query(Database::SELECT, $sql)
            ->parameters([
                ':teacherID'  => $teacherID,
                ':facultyID'  => $facultyID,
                ':milestone'  => $milestone,
                ':semesterID' => $semesterID,
            ])->execute();
    }

}