<?php defined('SYSPATH') or die('No direct script access.');

class Model_Teachers extends Model
{
    public static function getFaculties() {
        $sql = "CALL `GetFaculties`()";
        return DB::query(Database::SELECT, $sql)->execute()->as_array();
    }

    public static function getJobPositions() {
        $sql = "CALL `GetJobPositions`()";
        return DB::query(Database::SELECT, $sql)->execute()->as_array();
    }

    public function getTeachersByFaculty($facultyID) {
        $sql = "CALL `GetTeachersByFaculty`(:faculty)";
        return DB::query(Database::SELECT, $sql)
            ->param(':faculty', $facultyID)
            ->execute();
    }

    public static function byFaculty($facultyID) {
        $list = [];

        if ($facultyID > 0) {
            $teachers = self::getTeachersByFaculty($facultyID);

            foreach ($teachers as $row) {
                $row['DepartmentID'] = $row['DepID'];
                $row['DepartmentName'] = $row['DepName'];
                $list[] = $row;
            }
        }

        return $list;
    }

    public static function getTeachersByDepartment($departmentID) {
        $sql = "CALL `GetTeachersByDepartment`(:id)";
        return DB::query(Database::SELECT, $sql)
            ->param(':id', $departmentID)
            ->execute();
    }

    public static function byDepartment($departmentID) {
        $teachers = self::getTeachersByDepartment($departmentID);
        $list = [];

        foreach ($teachers as $row) {
            $row['DepartmentName'] = $row['DepName'];
            $list[] = $row;
        }

        return $list;
    }

    public static function getNamesForDiscipline($disciplineID, $asConcat = false, $asAbbreviation = false) {
        $teachers = Model_Discipline::load($disciplineID)->getTeachers();
        $list = [];

        foreach ($teachers as $row) {
            if ($asAbbreviation) {
                $row['FirstName'] = UTF8::substr($row['FirstName'], 0, 1) . '.';
                if (!empty($row['SecondName']))
                    $row['SecondName'] = UTF8::substr($row['SecondName'], 0, 1) . '.';
            }

            if ($asConcat)
                $list[] = $row['LastName'] . ' ' . $row['FirstName'] . ' ' . $row['SecondName'];
            else
                $list[] = $row;
        }

        return $list;
    }

    public static function searchTeachers($facultyID, $departmentID, $last, $first, $second) {
        $sql = "CALL `SearchTeachers`(:faculty, :department, :last, :first, :second)";
        return DB::query(Database::SELECT, $sql)
            ->parameters([
                ':faculty'    => $facultyID,
                ':department' => $departmentID,
                ':last'       => $last,
                ':first'      => $first,
                ':second'     => $second,
            ])->execute();
    }

    public static function searchTeacherNew($facultyID, $departmentID, $last, $first, $second, $disciplineID) {
        $sql = "CALL `SearchTeacherNew`(:faculty, :department, :last, :first, :second, :discipline)";
        return DB::query(Database::SELECT, $sql)
            ->parameters([
                ':faculty'    => $facultyID,
                ':department' => $departmentID,
                ':discipline' => $disciplineID,
                ':last'       => $last,
                ':first'      => $first,
                ':second'     => $second,
            ])->execute();
    }
}